import os, sys, re, jumpy
from subprocess import Popen, PIPE

version = '1.1'
print 'capture %s' % version

def addDevice(args, path=''):
	name = args[1]
	pms.addAction('%s%s' % (path, name), [sys.argv[0], 'play'] + args, '#monitor')

def readlines(cmd, pipe):
	p = Popen(cmd, stdout=(PIPE if pipe == 0 else None), stderr=(PIPE if pipe == 1 else None))
	lines = p.communicate()[pipe].splitlines()
	print '\n'.join(lines)
	return lines

class ffDevice:
	def getcmd(self, args):
		dev, name, input, size, fps = args
		OUTFILE=os.environ['OUTFILE']
		out_opts = os.environ['ffmpeg_out'] if 'ffmpeg_out' in os.environ else '-c:v mpeg2video -f mpegts -q:v 3'
		if 'libx264' in out_opts and not 'zerolatency' in out_opts:
			out_opts = out_opts + ' -tune zerolatency'
		return 'ffmpeg -y -rtbufsize 1500M -f {dev} -r {fps} -s {size} -i {input} {out_opts} -threads 0 {OUTFILE}' \
			.format(**locals())

class dshow (ffDevice):
	def explore(self):
		getdev = re.compile(r'\[dshow.+\] *((DirectShow|").+)')
		getopt = re.compile(r'\[dshow.+\] *pixel_format.* max s=(\w+) fps=(\w+)')

		lines = readlines(['ffmpeg', '-list_devices', 'true', '-f', 'dshow', '-i', 'dummy'], 1)
		devices = [m.group(1) for l in lines for m in [getdev.search(l)] if m]
		V = self.firstindexof('DirectShow video devices', devices)
		A = self.firstindexof('DirectShow audio devices', devices)
		video_devices = devices[V+1:A] if V > -1 else []
		audio_devices = devices[A+1:] if A > -1 else []
		print video_devices
		print audio_devices

		for v in video_devices:
			lines = readlines(['ffmpeg', '-f', 'dshow', '-list_options', 'true', '-i', 'video=%s' % v[1:-1]], 1)
			opts = list(set(
				[(m.group(1),m.group(2)) for l in lines for m in [getopt.search(l)] if m]
			))
			print 'opts: %s'%opts
			for size, fps in opts:
				name = '%s [%s@%sfps]' % (v[1:-1], size, fps)
				addDevice(['dshow', name, 'video=%s' % v, size, fps])
				for a in audio_devices:
					addDevice(['dshow', '%s + %s' % (name, a[1:-1]), \
						'audio=%s:video=%s' % (a, v), size, fps])
			print

	def firstindexof(self, s, l):
		for i,j in enumerate(l):
			if j.startswith(s): return i
		return -1

class vfw (ffDevice):
	def explore(self):
		pass #TODO

class v4l2 (ffDevice):
	def explore(self):
		pass #TODO

class x11grab (ffDevice):
	def explore(self):
		getdev = re.compile(r'(name of display|dimensions):\s+(\S+)')
		lines = readlines(['xdpyinfo'], 0)
		m = [m.group(2) for l in lines for m in [getdev.search(l)] if m]
		displays = [m[i:i+2] for i in range(0, len(m), 2)]
		print 'displays: %s' % displays
		for name, size in displays:
			addDevice(['x11grab', 'Display %s' % name, name, size, '15'])

if not (sys.platform.startswith('win32') or sys.platform.startswith('linux')):
	print 'Sorry, ffmpeg doesn\'t support device capture in %s. Exiting...' % sys.platform

elif len(sys.argv) == 1:
	pms.setInfo({
		'_title'  : 'Capture',
		'_desc'   : 'Grab and stream your devices.',
		'_icon'   : '#monitor+f=#888a85',
		'_version': str(version),
	})
	pms.addFolder('/Devices', [sys.argv[0], 'explore'])

elif sys.argv[1] == 'explore':
	if sys.platform.startswith('win32'):
		dshow().explore()
		vfw().explore()
	elif sys.platform.startswith('linux'):
		x11grab().explore()
		v4l2().explore()

elif sys.argv[1] == 'play':
	dev = eval(sys.argv[2])
	args = sys.argv[2:]
	cmd = dev().getcmd(args)
	pms.log( 'running: %s' % cmd)
	os.system(cmd)

# see:
# http://ffmpeg.org/ffmpeg-devices.html
# http://trac.ffmpeg.org/wiki/StreamingGuide
# http://trac.ffmpeg.org/wiki/DirectShow
# http://ffmpeg.org/trac/ffmpeg/wiki/How%20to%20grab%20the%20desktop%20%28screen%29%20with%20FFmpeg
# http://trac.ffmpeg.org/wiki/How%20to%20capture%20a%20webcam%20input
# http://ubuntuforums.org/showthread.php?t=1392026

# Windows directshow capture filters:
# x86 edition of the UScreenCapture DirectShow filter (video) from
# http://www.umediaserver.net/umediaserver/download.html
# and/or
# https://github.com/rdp/screen-capture-recorder-to-video-windows-free
# https://github.com/rdp/virtual-audio-capture-grabber-device
